/* defines.hpp

Copyright (c) 2010 - 2011 by Felix Lauer and Simon Schneegans

This program is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option)
any later version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
more details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>. */

# ifndef DEFINES_HPP_INCLUDED
# define DEFINES_HPP_INCLUDED

# define VERSION_MAJOR               "0"
# define VERSION_MINOR               "7"
# define VERSION_PATCH               "5"
# define VERSION_REVIS               "216"

# define SHIP_GRAPHICS_COUNT         15
# define SHIP_RADIUS                 18.f

# define SPACE_X_RESOLUTION          1440
# define SPACE_Y_RESOLUTION          810

# define INITIAL_WINDOW_X            960
# define INITIAL_WINDOW_Y            540

# define HOME_PLANET_RADIUS          100
# define SPACEOBJECT_MIN_RADIUS      50
# define SPACEOBJECT_MAX_RADIUS      120
# define SPACEOBJECT_MIN_GAP         150

# endif //DEFINES_HPP_INCLUDED
